/*
 * Copyright 2021 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  http://aws.amazon.com/apache2.0
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
*/

import 'package:amplify_authenticator/src/state/inherited_config.dart';
import 'package:amplify_authenticator/src/theme/amplify_theme.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class AmplifyProgressIndicator extends StatelessWidget {
  const AmplifyProgressIndicator({
    Key? key,
    this.primary = true,
  }) : super(key: key);

  final bool primary;

  @override
  Widget build(BuildContext context) {
    final ProgressIndicatorThemeData data;
    if (InheritedConfig.of(context).useAmplifyTheme) {
      data = primary
          ? AmplifyTheme.primaryProgressIndicatorTheme
          : AmplifyTheme.secondaryProgressIndicatorTheme;
    } else {
      data = ProgressIndicatorTheme.of(context);
    }
    return ProgressIndicatorTheme(
      data: data,
      child: const SizedBox(
        width: 16,
        height: 16,
        child: CircularProgressIndicator(),
      ),
    );
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties.add(DiagnosticsProperty<bool>('primary', primary));
  }
}
